@@ -1,25 +1,18 @@
 import copy
 import re
-import sys
 import types
 
 from .ucre import build_re
 
-if sys.version_info[1] == 6:
-    RE_TYPE = re._pattern_type
-elif sys.version_info[1] > 6:
-    RE_TYPE = re.Pattern
+# py>=37: re.Pattern, else: _sre.SRE_Pattern
+RE_TYPE = type(re.compile(r""))
 
 
-# =============================================================================
-# Helpers
-# =============================================================================
-def escape_re(string):
-    ESCAPE_RE = re.compile(r"[.?*+^$[\]\\(){}|-]")
-    return ESCAPE_RE.sub("\\$&", string)
+def _escape_re(string):
+    return re.sub(r"[.?*+^$[\]\\(){}|-]", "\\$&", string)
 
 
-def indexOf(text, search_value):
+def _index_of(text, search_value):
     try:
         result = text.index(search_value)
     except ValueError:
@@ -28,6 +21,14 @@ def indexOf(text, search_value):
     return result
 
 
+class SchemaError(Exception):
+    """Linkify schema error"""
+
+    def __init__(self, name, val):
+        message = "(LinkifyIt) Invalid schema '{}': '{}'".format(name, val)
+        super().__init__(message)
+
+
 class Match:
     """Match result.
 
@@ -38,6 +39,10 @@ class Match:
         raw (str): Matched string.
         text (str): Notmalized text of matched string.
         url (str): Normalized url of matched string.
+
+    Args:
+        linkifyit (:class:`linkify_it.main.LinkifyIt`) LinkifyIt object
+        shift (int): text searh position
     """
 
     def __repr__(self):
@@ -58,16 +63,6 @@ class Match:
         self.url = text
 
 
-# =============================================================================
-class SchemaError(Exception):
-    def __init__(self, name, val):
-        message = "(LinkifyIt) Invalid schema '{}': '{}'".format(name, val)
-        super().__init__(message)
-
-
-# =============================================================================
-
-
 class LinkifyIt:
     """Creates new linkifier instance with optional additional schemas.
 
@@ -76,50 +71,52 @@ class LinkifyIt:
     - ``http(s)://...`` , ``ftp://...``, ``mailto:...`` & ``//...`` links
     - "fuzzy" links and emails (example.com, foo@bar.com).
 
-    ``schemas`` is an object, where each key/value describes protocol/rule:
+    ``schemas`` is an dict where each key/value describes protocol/rule:
 
     - **key** - link prefix (usually, protocol name with ``:`` at the end, ``skype:``
       for example). `linkify-it` makes shure that prefix is not preceeded with
-      alphanumeric char and symbols. Only whitespaces and punctuation allowed.
+      alphanumeric char. Only whitespaces and punctuation allowed.
+
     - **value** - rule to check tail after link prefix
-      - *String* - just alias to existing rule
-      - *Object*
-        - *validate* - validator function (should return matched length on
-          success), or ``RegExp``.
+
+      - *str* - just alias to existing rule
+      - *dict*
+
+        - *validate* - either a ``re.Pattern``, ``re str`` (start with ``^``, and don't
+          include the link prefix itself), or a validator ``function`` which, given
+          arguments *self*, *text* and *pos* returns the length of a match in *text*
+          starting at index *pos*. *pos* is the index right after the link prefix.
         - *normalize* - optional function to normalize text & url of matched
           result (for example, for @twitter mentions).
 
-    ``options``:
+    ``options`` is an dict:
 
-    - **fuzzyLink** - recognige URL-s without ``http(s):`` prefix. Default ``true``.
+    - **fuzzyLink** - recognige URL-s without ``http(s):`` prefix. Default ``True``.
     - **fuzzyIP** - allow IPs in fuzzy links above. Can conflict with some texts
-      like version numbers. Default ``false``.
+      like version numbers. Default ``False``.
     - **fuzzyEmail** - recognize emails without ``mailto:`` prefix.
+    - **---** - set `True` to terminate link with `---` (if it's considered as long
+      dash).
 
-    Attributes:
+    Args:
         schemas (dict): Optional. Additional schemas to validate (prefix/validator)
-        options (dict): { fuzzyLink|fuzzyEmail|fuzzyIP: true|false }
+        options (dict): { fuzzy_link | fuzzy_email | fuzzy_ip: True | False }.
+            Default: {"fuzzy_link": True, "fuzzy_email": True, "fuzzy_ip": False}.
     """
 
-    # def build_default_schems(self, text, pos):
-    #     self.default_schemas["http:"] = self._validate_http(text, pos)
-    #     self.default_schemas["//"] = self._validate_double_slash(text, pos)
-    #     self.default_schemas["mailto:"] = self._validate_mailto(text, pos)
-
     def _validate_http(self, text, pos):
         tail = text[pos:]
         if not self.re.get("http"):
             # compile lazily, because "host"-containing variables can change on
             # tlds update.
-            self.re["http"] = re.compile(
+            self.re["http"] = (
                 "^\\/\\/"
                 + self.re["src_auth"]
                 + self.re["src_host_port_strict"]
-                + self.re["src_path"],
-                flags=re.IGNORECASE,
+                + self.re["src_path"]
             )
 
-        founds = self.re["http"].search(tail)
+        founds = re.search(self.re["http"], tail, flags=re.IGNORECASE)
         if founds:
             return len(founds.group())
 
@@ -131,22 +128,20 @@ class LinkifyIt:
         if not self.re.get("not_http"):
             # compile lazily, because "host"-containing variables can change on
             # tlds update.
-            self.re["not_http"] = re.compile(
-                "^" + self.re["src_auth"] +
-                # Don't allow single-level domains, because of false positives
-                # like '//test' with code comments
-                "(?:localhost|(?:(?:"
+            self.re["not_http"] = (
+                "^"
+                + self.re["src_auth"]
+                + "(?:localhost|(?:(?:"
                 + self.re["src_domain"]
                 + ")\\.)+"
                 + self.re["src_domain_root"]
                 + ")"
                 + self.re["src_port"]
                 + self.re["src_host_terminator"]
-                + self.re["src_path"],
-                flags=re.IGNORECASE,
+                + self.re["src_path"]
             )
 
-        founds = self.re["not_http"].search(tail)
+        founds = re.search(self.re["not_http"], tail, flags=re.IGNORECASE)
         if founds:
             if pos >= 3 and text[pos - 3] == ":":
                 return 0
@@ -162,12 +157,11 @@ class LinkifyIt:
         tail = text[pos:]
 
         if not self.re.get("mailto"):
-            self.re["mailto"] = re.compile(
-                "^" + self.re["src_email_name"] + "@" + self.re["src_host_strict"],
-                flags=re.IGNORECASE,
+            self.re["mailto"] = (
+                "^" + self.re["src_email_name"] + "@" + self.re["src_host_strict"]
             )
 
-        founds = self.re["mailto"].search(tail)
+        founds = re.search(self.re["mailto"], tail, flags=re.IGNORECASE)
         if founds:
             return len(founds.group(0))
 
@@ -180,7 +174,12 @@ class LinkifyIt:
     def _create_validator(self, regex):
         def func(text, pos):
             tail = text[pos:]
-            founds = regex.search(tail)
+            if isinstance(regex, str):
+                founds = re.search(regex, tail, flags=re.IGNORECASE)
+            else:
+                # re.Pattern
+                founds = re.search(regex, tail)
+
             if founds:
                 return len(founds.group(0))
 
@@ -253,9 +252,7 @@ class LinkifyIt:
         """Schemas compiler. Build regexps."""
 
         # Load & clone RE patterns.
-        # regex = copy.deepcopy(build_re(self._opts))
-        # self.re = copy.deepcopy(build_re(self._opts))
-        regex = self.re = build_re(self._opts)
+        self.re = build_re(self._opts)
 
         # Define dynamic patterns
         tlds = copy.deepcopy(self._tlds)
@@ -264,25 +261,20 @@ class LinkifyIt:
 
         if not self._tlds_replaced:
             tlds.append(self.tlds_2ch_src_re)
-        tlds.append(regex["src_xn"])
+        tlds.append(self.re["src_xn"])
 
-        regex["src_tlds"] = "|".join(tlds)
+        self.re["src_tlds"] = "|".join(tlds)
 
         def untpl(tpl):
-            return tpl.replace("%TLDS%", regex["src_tlds"])
+            return tpl.replace("%TLDS%", self.re["src_tlds"])
 
-        regex["email_fuzzy"] = re.compile(
-            untpl(regex["tpl_email_fuzzy"]), flags=re.IGNORECASE
-        )
-        regex["link_fuzzy"] = re.compile(
-            untpl(regex["tpl_link_fuzzy"]), flags=re.IGNORECASE
-        )
-        regex["link_no_ip_fuzzy"] = re.compile(
-            untpl(regex["tpl_link_no_ip_fuzzy"]), flags=re.IGNORECASE
-        )
-        regex["host_fuzzy_test"] = re.compile(
-            untpl(regex["tpl_host_fuzzy_test"]), flags=re.IGNORECASE
-        )
+        self.re["email_fuzzy"] = untpl(self.re["tpl_email_fuzzy"])
+
+        self.re["link_fuzzy"] = untpl(self.re["tpl_link_fuzzy"])
+
+        self.re["link_no_ip_fuzzy"] = untpl(self.re["tpl_link_no_ip_fuzzy"])
+
+        self.re["host_fuzzy_test"] = untpl(self.re["tpl_host_fuzzy_test"])
 
         #
         # Compile each schema
@@ -304,6 +296,8 @@ class LinkifyIt:
             if isinstance(val, dict):
                 if isinstance(val.get("validate"), RE_TYPE):
                     compiled["validate"] = self._create_validator(val.get("validate"))
+                elif isinstance(val.get("validate"), str):
+                    compiled["validate"] = self._create_validator(val.get("validate"))
                 elif isinstance(val.get("validate"), types.MethodType):
                     compiled["validate"] = val.get("validate")
                 # Add custom handler
@@ -311,6 +305,8 @@ class LinkifyIt:
                     setattr(LinkifyIt, "func", val.get("validate"))
                     compiled["validate"] = self.func
                 else:
+                    print(RE_TYPE)
+                    print(val.get("validate"), type(val.get("validate")))
                     raise SchemaError(name, val)
 
                 if isinstance(val.get("normalize"), types.MethodType):
@@ -354,23 +350,22 @@ class LinkifyIt:
         #
         slist = "|".join(
             [
-                escape_re(name)
+                _escape_re(name)
                 for name, val in self._compiled.items()
                 if len(name) > 0 and val
             ]
         )
 
         re_schema_test = (
-            "(^|(?!_)(?:[><\uff5c]|" + regex["src_ZPCc"] + "))(" + slist + ")"
+            "(^|(?!_)(?:[><\uff5c]|" + self.re["src_ZPCc"] + "))(" + slist + ")"
         )
 
         # (?!_) cause 1.5x slowdown
-        self.re["schema_test"] = re.compile(re_schema_test, flags=re.IGNORECASE)
-        self.re["schema_search"] = re.compile(re_schema_test, flags=re.IGNORECASE)
+        self.re["schema_test"] = re_schema_test
+        self.re["schema_search"] = re_schema_test
 
-        self.re["pretest"] = re.compile(
-            "(" + re_schema_test + ")|(" + self.re["host_fuzzy_test"].pattern + ")|@",
-            flags=re.IGNORECASE,
+        self.re["pretest"] = (
+            "(" + re_schema_test + ")|(" + self.re["host_fuzzy_test"] + ")|@"
         )
 
         # Cleanup
@@ -378,36 +373,49 @@ class LinkifyIt:
         self._reset_scan_cache()
 
     def add(self, schema, definition):
-        """Add new rule definition.
+        """Add new rule definition. (chainable)
 
-        See constructor description for details.
+        See :class:`linkify_it.main.LinkifyIt` init description for details.
+        ``schema`` is a link prefix (``skype:``, for example), and ``definition``
+        is a ``str`` to alias to another schema, or an ``dict`` with ``validate`` and
+        optionally `normalize` definitions. To disable an existing rule, use
+        ``.add(<schema>, None)``.
 
         Args:
             schema (str): rule name (fixed pattern prefix)
-            definition (str or regex or object): schema definition
+            definition (`str` or `re.Pattern`): schema definition
+
+        Return:
+            :class:`linkify_it.main.LinkifyIt`
         """
         self._schemas[schema] = definition
         self._compile()
         return self
 
     def set(self, options):
-        """Set recognition options for links without schema.
+        """Override default options. (chainable)
+
+        Missed properties will not be changed.
 
         Args:
-            options (object): { fuzzyLink|fuzzyEmail|fuzzyIP: true|false }
+            options (dict): ``keys``: [``fuzzy_link`` | ``fuzzy_email`` | ``fuzzy_ip``].
+                ``values``: [``True`` | ``False``]
+
+        Return:
+            :class:`linkify_it.main.LinkifyIt`
         """
         self._opts.update(options)
         return self
 
     def test(self, text):
-        """Searches linkifiable pattern and returns `true` on success or `false`
+        """Searches linkifiable pattern and returns ``True`` on success or ``False``
         on fail.
 
         Args:
-            text (str): xxxxxx
+            text (str): text to search
 
         Returns:
-            bool: xxxxxx
+            bool: ``True`` if a linkable pattern was found, otherwise it is ``False``.
         """
         self._text_cache = text
         self._index = -1
@@ -415,10 +423,10 @@ class LinkifyIt:
         if not len(text):
             return False
 
-        if self.re["schema_test"].search(text):
+        if re.search(self.re["schema_test"], text, flags=re.IGNORECASE):
             regex = self.re["schema_search"]
             last_index = 0
-            matched_iter = regex.finditer(text[last_index:])
+            matched_iter = re.finditer(regex, text[last_index:], flags=re.IGNORECASE)
             for matched in matched_iter:
                 last_index = matched.end(0)
                 m = (matched.group(), matched.groups()[0], matched.groups()[1])
@@ -431,7 +439,9 @@ class LinkifyIt:
 
         if self._opts.get("fuzzy_link") and self._compiled.get("http:"):
             # guess schemaless links
-            matched_tld = self.re["host_fuzzy_test"].search(text)
+            matched_tld = re.search(
+                self.re["host_fuzzy_test"], text, flags=re.IGNORECASE
+            )
             if matched_tld:
                 tld_pos = matched_tld.start(0)
             else:
@@ -444,7 +454,7 @@ class LinkifyIt:
                     else:
                         pattern = self.re["link_no_ip_fuzzy"]
 
-                    ml = pattern.search(text)
+                    ml = re.search(pattern, text, flags=re.IGNORECASE)
                     if ml:
                         shift = ml.start(0) + len(ml.groups()[0])
 
@@ -455,11 +465,11 @@ class LinkifyIt:
 
         if self._opts.get("fuzzy_email") and self._compiled.get("mailto:"):
             # guess schemaless emails
-            at_pos = indexOf(text, "@")
+            at_pos = _index_of(text, "@")
             if at_pos >= 0:
                 # We can't skip this check, because this cases are possible:
                 # 192.168.1.1@gmail.com, my.in@example.com
-                me = self.re["email_fuzzy"].search(text)
+                me = re.search(self.re["email_fuzzy"], text, flags=re.IGNORECASE)
                 if me:
                     shift = me.start(0) + len(me.groups()[0])
                     next_shift = me.start(0) + len(me.group())
@@ -482,47 +492,48 @@ class LinkifyIt:
         when you need to check that link NOT exists.
 
         Args:
-            text (str): xxxxxx
+            text (str): text to search
 
         Returns:
-            bool: xxxxxx
+            bool: ``True`` if a linkable pattern was found, otherwise it is ``False``.
         """
-        if self.re["pretest"].search(text):
+        if re.search(self.re["pretest"], text, flags=re.IGNORECASE):
             return True
 
         return False
 
     def test_schema_at(self, text, name, position):
-        """Similar to `~linkify_it.LinkifyIt.test` but checks only specific protocol
-        tail exactly at given position. Returns length of found pattern (0 on fail).
+        """Similar to :meth:`linkify_it.main.LinkifyIt.test` but checks only
+        specific protocol tail exactly at given position.
 
         Args:
             text (str): text to scan
             name (str): rule (schema) name
-            position (int): text offset to check from
+            position (int): length of found pattern (0 on fail).
+
+        Returns:
+            int: text (str): text to search
         """
         # If not supported schema check requested - terminate
         if not self._compiled.get(name.lower()):
             return 0
         return self._compiled.get(name.lower()).get("validate")(text, position)
-        # custom twitter API
-        # return self._compiled.get(name.lower()).get("validate")(self, text, position)
 
     def match(self, text):
-        """Returns array of found link descriptions or `null` on fail.
+        """Returns ``list`` of found link descriptions or ``None`` on fail.
 
-        We strongly recommend to use `~linkify_it.LinkifyIt.test` first, for best
-        speed.
+        We strongly recommend to use :meth:`linkify_it.main.LinkifyIt.test`
+        first, for best speed.
 
         Args:
-            text (str):
+            text (str): text to search
 
         Returns:
-            list or None: Result match description
-                * **schema** - link schema, can be empty for fuzzy links, or `//`
-                    for protocol-neutral  links.
+            ``list`` or ``None``: Result match description:
+                * **schema** - link schema, can be empty for fuzzy links, or ``//``
+                  for protocol-neutral  links.
                 * **index** - offset of matched text
-                * **lastIndex** - offset of matched text
+                * **last_index** - offset of matched text
                 * **raw** - offset of matched text
                 * **text** - normalized text
                 * **url** - link, generated from matched text
@@ -551,26 +562,26 @@ class LinkifyIt:
         return None
 
     def tlds(self, list_tlds, keep_old=False):
-        """Load (or merge) new tlds list.
+        """Load (or merge) new tlds list. (chainable)
 
         Those are user for fuzzy links (without prefix) to avoid false positives.
         By default this algorythm used:
 
         * hostname with any 2-letter root zones are ok.
         * biz|com|edu|gov|net|org|pro|web|xxx|aero|asia|coop|info|museum|name|shop|рф
-            are ok.
+          are ok.
         * encoded (`xn--...`) root zones are ok.
 
         If list is replaced, then exact match for 2-chars root zones will be checked.
 
         Args:
-            list_tlds (list): list of tlds
-            keep_old (bool): merge with current list if `true` (`false` by default)
+            list_tlds (list or str): ``list of tlds`` or ``tlds string``
+            keep_old (bool): merge with current list if q`True`q (q`Falseq` by default)
         """
         _list = list_tlds if isinstance(list_tlds, list) else [list_tlds]
 
         if not keep_old:
-            self._tlds = copy.copy(_list)
+            self._tlds = _list
             self._tlds_replaced = True
             self._compile()
             return self
